/**************************************************************************//**
 * @file     DIV.h
 * @version  V2.1
 * $Revision: 3 $
 * $Date: 14/02/11 7:24p $
 * @brief    M051 Series DIV Driver Header File
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __DIVIDER_H__
#define __DIVIDER_H__


/**
 * @brief      Divison function to calculate (x/y)
 *
 * @param[in]  x the dividend of the divison
 * @param[in]  y the divisor of the divison
 *
 * @return     The result of (x/y)
 *
 * @details    This is a divison function to calculate x/y
 *
 */
static __INLINE int32_t DIV_Div(int32_t x, int16_t y)
{
    uint32_t *p32;

    p32 = (uint32_t *)DIV_BASE;
    *p32++ = x;
    *p32++ = y;
    return *p32;
}


/**
 * @brief      To calculate the remainder of x/y, i.e., the result of x mod y.
 *
 * @param[in]  x the dividend of the divison
 * @param[in]  y the divisor of the divison
 *
 * @return     The remainder of (x/y)
 *
 * @details    This function is used to calculate the remainder of x/y.
 */
static __INLINE int16_t DIV_Mod(int32_t x, int16_t y)
{
    uint32_t *p32;

    p32 = (uint32_t *)DIV_BASE;
    *p32++ = x;
    *p32++ = y;
    return p32[1];
}


#endif


